import asyncio
import functools as _functools


def partial(func, *args, **keywords):
    """Wrapper around the standard library functools partial,
    this adds support for async functions."""
    # In case somehow we don't maintain feature parity and
    # someone uses it on a traditional func
    if not asyncio.iscoroutinefunction(func):
        return _functools.partial(func, *args, **keywords)

    if hasattr(func, 'func'):
        args = func.args + args
        tmpkw = func.keywords.copy()
        tmpkw.update(keywords)
        keywords = tmpkw
        del tmpkw
        func = func.func

    async def newfunc(*fargs, **fkeywords):
        newkeywords = keywords.copy()
        newkeywords.update(fkeywords)
        return await func(*(args + fargs), **newkeywords)

    newfunc.func = func
    newfunc.args = args
    newfunc.keywords = keywords
    return newfunc




from setuptools import setup, find_packages

__version__ = '0.0.1'

setup(name='asynclib',
      description='Utlity library for working with async python, including'
                  ' some stdlib replacements to be compatible with native '
                  'coroutines.',
      author='Matt Rasband, Nick Humrich',
      author_email='matt.rasband@gmail.com',
      license='Apache-2.0',
      url='https://github.com/wickedasp/asynclib',
      download_url=('https://github.com/wickedasp/asynclib'
                    '/releases/v' + __version__ + '.tar.gz'),
      keywords=(
          'asyncio',
      ),
      packages=find_packages(),
      classifiers=[
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3 :: Only',
          'License :: OSI Approved :: Apache Software License',
          'Intended Audience :: Developers',
          'Development Status :: 2 - Pre-Alpha',
          'Topic :: Software Development',
      ],
      setup_requires=[],
      install_requires=[],
      extras_require={},
      tests_require=[],
      entry_points={},
      zip_safe=False)
